package de.noahpeeters.gameoflife.classic;

import de.noahpeeters.gameoflife.classic.model.Board;
import de.noahpeeters.gameoflife.classic.model.CellPosition;

import java.util.HashMap;
import java.util.HashSet;

public class BoardController {
    private Board board;
    private final HashMap<CellPosition, Integer> aliveStatusCounts;

    public BoardController() {
        this.board = new Board(new HashSet<>());
        this.aliveStatusCounts = new HashMap<>();

        this.board.getAliveCells().add(new CellPosition(1, 0));
        this.board.getAliveCells().add(new CellPosition(2, 1));
        this.board.getAliveCells().add(new CellPosition(0, 2));
        this.board.getAliveCells().add(new CellPosition(1, 2));
        this.board.getAliveCells().add(new CellPosition(2, 2));

        this.board.getAliveCells().forEach(cell -> aliveStatusCounts.put(cell, 1));
    }

    public void step() {
        Board newBoard = new Board(new HashSet<>(board.getAliveCells()));
        for (CellPosition position: board.getActivePositions()) {
            long aliveNeighbourCount = board.getNeighboursOfPosition(position).stream().filter(board.getAliveCells()::contains).count();
            boolean isAlive = board.getAliveCells().contains(position);

            if (isAlive && (aliveNeighbourCount < 2 || aliveNeighbourCount > 3)) {
                // dead
                newBoard.getAliveCells().remove(position);
            } else if (!isAlive && aliveNeighbourCount == 3) {
                // alive +1 if dead before
                newBoard.getAliveCells().add(position);
                aliveStatusCounts.compute(position, (key, value) -> value == null ? 1 : value + 1);
            }
        }
        this.board = newBoard;
    }

    public void print() {
        for (int y = 0; y < 10; y++) {
            for (int x = 0; x < 10; x++) {
                CellPosition cellPosition = new CellPosition(x, y);
                System.out.print(board.getAliveCells().contains(cellPosition) ? "X" : " ");
            }
            System.out.println();
        }

        System.out.println("Alive cells: " + board.getAliveCells().size());

        System.out.println("----------");

        aliveStatusCounts.forEach((key, value) -> System.out.println(key.toString() + ": " + value));

        System.out.println("----------");
    }
}
