package de.noahpeeters.gameoflife.classic;

import de.noahpeeters.gameoflife.classic.model.Board;
import de.noahpeeters.gameoflife.classic.model.CellPosition;
import java.util.HashMap;
import java.util.HashSet;

public class BoardController {
    private Board board;
    private HashMap<CellPosition, Integer> aliveMap;

    public BoardController() {
        aliveMap = new HashMap<>();
        this.board = new Board(new HashSet<>());

        this.board.getAliveCells().add(new CellPosition(1, 0));
        this.board.getAliveCells().add(new CellPosition(2, 1));
        this.board.getAliveCells().add(new CellPosition(0, 2));
        this.board.getAliveCells().add(new CellPosition(1, 2));
        this.board.getAliveCells().add(new CellPosition(2, 2));
    }

    public void step() {
        Board newBoard = new Board(new HashSet<>(board.getAliveCells()));
        for (CellPosition position: board.getActivePositions()) {
            long aliveNeighbourCount = board.getNeighboursOfPosition(position).stream().filter(board.getAliveCells()::contains).count();
            boolean isAlive = board.getAliveCells().contains(position);

            if (isAlive && (aliveNeighbourCount < 2 || aliveNeighbourCount > 3)) {
                newBoard.getAliveCells().remove(position);
            } else if (!isAlive && aliveNeighbourCount == 3) {
                newBoard.getAliveCells().add(position);
                if (!this.board.getAliveCells().contains(position)) {
                    aliveMap.put(position, aliveMap.getOrDefault(position, 0) + 1);
                }
            }
        }
        this.board = newBoard;
    }

    public void print() {
        for (int y = 0; y < 10; y++) {
            for (int x = 0; x < 10; x++) {
                CellPosition cellPosition = new CellPosition(x, y);
                System.out.print(board.getAliveCells().contains(cellPosition) ? "X" : " ");
            }
            System.out.println();
        }
        System.out.println("----------");
        System.out.println(board.aliveCount());
        System.out.println("0/0: " + aliveMap.getOrDefault(new CellPosition(0,0),0));
        System.out.println("3/2: " + aliveMap.getOrDefault(new CellPosition(3,2), 0));
    }
}
